<?php
/**
 * A generic filter for LFDB2 project
 *
 * @author		Roland Haeder <webmaster@shipsimu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2013 LFDB2 Developer Team
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.shipsimu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
class BaseLfdb2Filter extends BaseFilter {
	/**
	 * Array with all data XML nodes (which hold the actual data) and their values
	 */
	protected $dataXmlNodes = array();

	/**
	 * Protected constructor
	 *
	 * @param	$className	Real name of class
	 * @return	void
	 */
	protected function __construct ($className) {
		// Call parent constructor
		parent::__construct($className);
	}

	/**
	 * Processes the given raw message content. The method renderXmlContent
	 * may throw (not the method itself) several exceptions:
	 *
	 * InvalidXmlNodeException  - If an invalid XML node has been found (e.g.
	 *                            wrong/out-dated template used)
	 * XmlNodeMismatchException - Again might be caused by invalid XML node
	 *                            usage
	 * XmlParserException       - If the XML message is damaged or not
	 *                            well-formed
	 *
	 * @param	$messageType		Type of message
	 * @param	$messageContent		Raw message content
	 * @param	$packageInstance	An instance of a Receivable class
	 * @return	void
	 * @todo	Exceptions from renderXmlContent() are currently unhandled
	 */
	protected function genericProcessMessage ($messageType, $messageContent, Receivable $packageInstance) {
		$this->partialStub('Please fix this imported code.');
		die();

		// Get a template instance from the factory
		$templateInstance = XmlTemplateEngineFactory::createXmlTemplateEngineInstance('node_' . $messageType . '_template_class');

		// And render the XML content (aka message)
		$templateInstance->renderXmlContent($messageContent);

		// Debug message
		//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput(str_replace('_', '-', strtoupper($messageType)) . '-TAG: Handling ' . strlen($messageContent) . ' bytes: ' . $messageContent);

		/*
		 * The template system now stores all required data as 'general'
		 * variables, so simply get them. If there is an invalid XML node
		 * inside the message, the above method call will cause exceptions.
		 */
		foreach ($this->dataXmlNodes as $key => $dummy) {
			// Call it
			$value = $templateInstance->readXmlData($key);

			/*
			 * If value is NULL, a variable hasn't been found. This could mean
			 * that *this* node is running an out-dated software or the other
			 * peer is using an out-dated $messageType.xml template.
			 */
			if (is_null($value)) {
				// Output a warning
				self::createDebugInstance(__CLASS__)->debugOutput(str_replace('_', '-', strtoupper($messageType)) . '-TAG: Found not fully supported variable ' . $key . ' - skipping.');

				// Skip this part, don't write NULLs to the array
				continue;
			} // END - if

			// Debug message
			//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__)->debugOutput(str_replace('_', '-', strtoupper($messageType)) . '-TAG: key=' . $key . ',value=' . $value);

			// Set it now
			$this->dataXmlNodes[$key] = $value;
		} // END - foreach

		// Construct an array for pushing it on next stack
		$messageArray = array(
			// Message data itself
			NetworkPackage::MESSAGE_ARRAY_DATA => $this->dataXmlNodes,
			// Message type (which is $messageType)
			NetworkPackage::MESSAGE_ARRAY_TYPE => $messageType
		);

		// Push the processed message back on stack
		$packageInstance->getStackerInstance()->pushNamed(NetworkPackage::STACKER_NAME_PROCESSED_MESSAGE, $messageArray);
	}
}

// [EOF]
?>
